import java.rmi.RemoteException;

import org.apache.axis2.Constants;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.AxisFault;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import javax.activation.DataHandler;
import javax.activation.FileDataSource;

import com.ebay.www.marketplace.services.FileTransferServiceStub;
import org.apache.axis2.databinding.types.Token;

/*
 * FTSServiceClient wrap up the service stub class to make invocation easier
 */
public class FTSServiceClient extends FileTransferServiceStub {
	
	private FTSServiceClient(String endPointURL)
			throws org.apache.axis2.AxisFault {
		super(endPointURL);
	}
	
	/*
	 * This method is used to pretty print the ErrorMessage object returned from
	 * the service response.
	 */
	private static String printErrors(ErrorMessage errorMessage) {
		System.out.println("Error messages returned from the service call are:");
		StringBuilder builder = new StringBuilder("");
		if (errorMessage != null) {
			ErrorData[] errors = errorMessage.getError();
			for (ErrorData ed : errors) {
				builder.append(ed.getMessage());
				builder.append("\n");
			}
		}
		return builder.toString();
	}

	/*
	 * This method represent the downloadFile service operation.
	 */
	public static boolean downloadFile(String fileSaveToPath, String task_ref_id, String file_ref_id)
			throws AxisFault, RemoteException, Exception {
		String authToken=LMSProperties.getInstance().getAuthToken();
		String endPointURL=LMSProperties.getInstance().getFtsEndPointURL();
		FTSServiceClient testStub = new FTSServiceClient(endPointURL);
		testStub._getServiceClient().getOptions().setProperty(
				Constants.Configuration.ENABLE_MTOM, Constants.VALUE_TRUE);
		testStub._getServiceClient().getOptions().setTimeOutInMilliSeconds(
				LMSProperties.getInstance().getTimeOutInMilliSeconds());

		//putin HTTP Headers
		MessageContext msgCtx = new MessageContext();
		testStub.addHttpHeader(msgCtx, "X-EBAY-SOA-OPERATION-NAME",
				"downloadFile");
		testStub.addHttpHeader(msgCtx, "X-EBAY-SOA-SECURITY-TOKEN", authToken);
		testStub
				.addHttpHeader(msgCtx, "X-EBAY-SOA-SERVICE-NAME",
						"{http://www.ebay.com/marketplace/services}FileTransferService");
		testStub.addHttpHeader(msgCtx, "X-EBAY-SOA-MESSAGE-PROTOCOL", "SOAP12");
		testStub
				.addHttpHeader(msgCtx, "X-EBAY-SOA-REQUEST-DATA-FORMAT", "SOAP");
		testStub
				.addHttpHeader(msgCtx, "X-EBAY-SOA-RESPONSE-DATA-FORMAT", "XML");

		//prepare the requst and invoke
		DownloadFileRequestE dReq = new DownloadFileRequestE();
		DownloadFileRequest downloadReq = new DownloadFileRequest();
		downloadReq.setTaskReferenceId(task_ref_id);
		downloadReq.setFileReferenceId(file_ref_id);
		dReq.setDownloadFileRequest(downloadReq);
		DownloadFileResponseE dRsp = testStub
				.downloadFile(dReq, msgCtx);
		if (dRsp == null) {
			System.out
					.println("The downloadFile service operation returned a null response ");
			return false;
		}
		DownloadFileResponse downloadRsp = dRsp
				.getDownloadFileResponse();

		AckValue ackValue = downloadRsp.getAck();

		System.out.println("Ack=" + ackValue.getValue());

		if ("Success".equals(ackValue.getValue())) {
			FileAttachment attachment = downloadRsp.getFileAttachment();

			DataHandler dh = attachment.getData();
			try {
				InputStream in = dh.getInputStream();
				BufferedInputStream bis = new BufferedInputStream(in);

				FileOutputStream fo = new FileOutputStream(new File(
						fileSaveToPath)); // "C:/myDownLoadFile.gz"
				BufferedOutputStream bos = new BufferedOutputStream(fo);
				int bytes_read = 0;
				byte[] dataBuf = new byte[4096];
				while ((bytes_read = bis.read(dataBuf)) != -1) {
					bos.write(dataBuf, 0, bytes_read);
				}
				bis.close();
				bos.flush();
				bos.close();

				System.out.println("File attachment has been saved successfully to "
						+ fileSaveToPath);

			} catch (Exception e) {
				e.printStackTrace();
				System.out
						.println("\nException caught while trying to save the attachement.");
				testStub = null;
				return false;
			}
		} else {
			ErrorMessage msg = downloadRsp.getErrorMessage();
			System.out.println("\n"+printErrors(msg));
			return false;
		}
		testStub = null;
		return true;
	}

	/*
	 * This method represent the uploadFile service operation
	 */
	public static boolean uploadFile(String fileFormat, String task_ref_id,
			String file_ref_id, String fileToUpload) throws AxisFault,
			RemoteException {
		String authToken=LMSProperties.getInstance().getAuthToken();
		String endPointURL=LMSProperties.getInstance().getFtsEndPointURL();
		FTSServiceClient testStub = new FTSServiceClient(endPointURL);
		testStub._getServiceClient().getOptions().setProperty(
				Constants.Configuration.ENABLE_MTOM, Constants.VALUE_TRUE);
		testStub._getServiceClient().getOptions().setTimeOutInMilliSeconds(
				LMSProperties.getInstance().getTimeOutInMilliSeconds());

		MessageContext msgCtx = new MessageContext();
		testStub.addHttpHeader(msgCtx, "X-EBAY-SOA-OPERATION-NAME",
				"uploadFile");
		testStub.addHttpHeader(msgCtx, "X-EBAY-SOA-SECURITY-TOKEN", authToken);
		testStub
				.addHttpHeader(msgCtx, "X-EBAY-SOA-SERVICE-NAME",
						"{http://www.ebay.com/marketplace/services}FileTransferService");
		testStub.addHttpHeader(msgCtx, "X-EBAY-SOA-MESSAGE-PROTOCOL", "SOAP12");
		testStub
				.addHttpHeader(msgCtx, "X-EBAY-SOA-REQUEST-DATA-FORMAT", "SOAP");
		testStub
				.addHttpHeader(msgCtx, "X-EBAY-SOA-RESPONSE-DATA-FORMAT", "XML");
		
		UploadFileRequestE uReq = new UploadFileRequestE();
		UploadFileRequest uploadReq = new UploadFileRequest();
		uploadReq.setFileFormat(new Token(fileFormat)); 
		// file format only accepts
		// either "gzip" or
		// "zip"
		uploadReq.setTaskReferenceId(task_ref_id);
		uploadReq.setFileReferenceId(file_ref_id);

		FileAttachment attachment = new FileAttachment();
		DataHandler dh = new DataHandler(new FileDataSource(new File(
				fileToUpload)));
		attachment.setData(dh);
		uploadReq.setFileAttachment(attachment);
		uReq.setUploadFileRequest(uploadReq);
		UploadFileResponseE uRsp = testStub
				.uploadFile(uReq, msgCtx);
		System.out.println("Uploading file " + fileToUpload);
		if (uRsp == null) {
			System.out
					.println("The uploadFile service operation returned a null response ");
			return false;
		}
		UploadFileResponse uploadRsp = uRsp
				.getUploadFileResponse();

		AckValue ackValue = uploadRsp.getAck();

		System.out.println("Ack=" + ackValue.getValue());

		if ("Success".equals(ackValue.getValue())) {
			System.out.println("File uploading complete.");
		} else {
			ErrorMessage msg = uploadRsp.getErrorMessage();
			System.out.println(printErrors(msg));
			return false;
		}
		testStub = null;
		return true;

	}

	/*
	 * main method that is just for quick testing purpose.
	 */
	public static void main(String[] args) throws Exception {
		downloadFile("D:/myDownLoadFile.zip", "50000000874", "50000000804");

	}

}
