import java.io.FileInputStream;
import java.io.IOException;
import java.util.Properties;

/*
 * BDXProperties is a singleton class, it reads the BDX.properties file,
 * and then keep the properties in its singleton instance.
 */
public class LMSProperties {
	private String bdxEndPointURL;
	private String ftsEndPointURL;
	private String authToken;
	private long timeOutInMilliSeconds;
	private int getJobStatusQueryInterval;
    /**
     * singleton
     * 
     */
    private static LMSProperties instance = new LMSProperties();

    private LMSProperties() {
    	Properties properties = new Properties();
    	String propertiesFilePath = getProjectPath()+"\\LMS.properties";
    	System.out.println("properties read from "+propertiesFilePath);
	    try {
	        properties.load(new FileInputStream(propertiesFilePath));
	    } catch (IOException e) {
	    	System.out.println("[LMSProperties] error: "+e.toString());
	    }
	    bdxEndPointURL=properties.getProperty("bdxEndPointURL");
	    ftsEndPointURL=properties.getProperty("ftsEndPointURL");
	    authToken=properties.getProperty("authToken");
	    if (bdxEndPointURL==null || bdxEndPointURL.length()==0)
	    	System.out.println("\n[LMSProperties] error: bdxEndPointURL should not be null\n");
	    if (ftsEndPointURL==null || ftsEndPointURL.length()==0)
	    	System.out.println("\n[LMSProperties] error: ftsEndPointURL should not be null\n");
	    if (authToken==null || authToken.length()==0 )
	    	System.out.println("\n[LMSProperties] error: authToken should not be null\n");
	    
	    timeOutInMilliSeconds = Long.parseLong(properties.getProperty("timeOutInMilliSeconds"));
	    getJobStatusQueryInterval = Integer.parseInt(properties.getProperty("getJobStatusQueryInterval"));
    }
    /**
     * singleton reference
     * 
     */
    public static LMSProperties getInstance() {
        return instance;
    }
	public String getAuthToken() {
		return authToken;
	}
	public String getBdxEndPointURL() {
		return bdxEndPointURL;
	}
	public String getFtsEndPointURL() {
		return ftsEndPointURL;
	}
	public long getTimeOutInMilliSeconds() {
		return timeOutInMilliSeconds;
	}
	public int getGetJobStatusQueryInterval() {
		return getJobStatusQueryInterval;
	}
	
	private String getProjectPath() {
		String path = null;
		try{
			path = new java.io.File(".").getCanonicalPath();
		}catch(IOException e){
			System.out.println("Could not get the root path of the project, this is not a valid Unit Test run.");
		}
		return path;
	}
	
}
